@@ -1,5 +1,5 @@
 package Future::IO::Impl::Uring;
-$Future::IO::Impl::Uring::VERSION = '0.004';
+$Future::IO::Impl::Uring::VERSION = '0.005';
 use 5.020;
 use warnings;
 use experimental 'signatures';
@@ -7,14 +7,14 @@ use experimental 'signatures';
 use parent 'Future::IO::ImplBase';
 __PACKAGE__->APPLY;
 
-use IO::Uring qw/IOSQE_ASYNC IORING_TIMEOUT_ABS IORING_TIMEOUT_REALTIME IORING_TIMEOUT_ETIME_SUCCESS P_PID P_PGID P_ALL WEXITED/;
+use IO::Uring qw/IORING_TIMEOUT_ABS IORING_TIMEOUT_REALTIME IORING_TIMEOUT_ETIME_SUCCESS P_PID P_PGID P_ALL WEXITED/;
 use Errno 'ETIME';
 use Signal::Info qw/CLD_EXITED/;
 use Time::Spec;
 use IO::Socket;
 use IO::Poll qw/POLLIN POLLOUT/;
 
-my $ring = IO::Uring->new(128);
+my $ring = $Future::Uring::ring // IO::Uring->new(128);
 
 sub accept($self, $fh) {
 	my $future = Future::IO::Impl::Uring::_Future->new;
@@ -62,28 +62,14 @@ sub connect($self, $fh, $name) {
 	return $future;
 }
 
-sub ready_for_read($self, $fh) {
+sub poll($self, $fh, $mask) {
 	my $future = Future::IO::Impl::Uring::_Future->new;
-	my $id = $ring->poll($fh, POLLIN, 0, sub($res, $flags) {
+	my $id = $ring->poll($fh, $mask, 0, sub($res, $flags) {
 		if ($res >= 0) {
-			$future->done;
+			$future->done($res);
 		} else {
 			local $! = -$res;
-			$future->fail("ready_for_read: $!\n", ready_for_read => $fh, $!);
-		}
-	});
-	$future->on_cancel(sub { $ring->cancel($id, 0, 0) });
-	return $future;
-}
-
-sub ready_for_write($self, $fh) {
-	my $future = Future::IO::Impl::Uring::_Future->new;
-	my $id = $ring->poll($fh, POLLOUT, 0, sub($res, $flags) {
-		if ($res >= 0) {
-			$future->done;
-		} else {
-			local $! = -$res;
-			$future->fail("ready_for_write: $!\n", ready_for_write => $fh, $!);
+			$future->fail("poll: $!\n", poll => $fh, $!);
 		}
 	});
 	$future->on_cancel(sub { $ring->cancel($id, 0, 0) });
@@ -94,7 +80,7 @@ sub recv($self, $fh, $length, $flags) {
 	my $future = Future::IO::Impl::Uring::_Future->new;
 	my $buffer = "\0" x $length;
 	$flags //= 0;
-	my $id = $ring->recv($fh, $buffer, $flags, 0, IOSQE_ASYNC, sub($res, $flags) {
+	my $id = $ring->recv($fh, $buffer, $flags, 0, 0, sub($res, $flags) {
 		if ($res > 0) {
 			$future->done($res == $length ? $buffer : substr($buffer, 0, $res));
 		} elsif ($res == 0) {
@@ -121,9 +107,9 @@ sub send($self, $fh, $buffer, $flags, $to) {
 	$flags //= 0;
 	my $id;
 	if (defined $to) {
-		$id = $ring->sendto($fh, $buffer, $flags, $to, 0, IOSQE_ASYNC, $callback);
+		$id = $ring->sendto($fh, $buffer, $flags, $to, 0, 0, $callback);
 	} else {
-		$id = $ring->send($fh, $buffer, $flags, 0, IOSQE_ASYNC, $callback);
+		$id = $ring->send($fh, $buffer, $flags, 0, 0, $callback);
 	}
 	$future->on_cancel(sub { $ring->cancel($id, 0, 0) });
 	return $future;
@@ -148,7 +134,7 @@ sub sleep($self, $seconds) {
 sub sysread($self, $fh, $length) {
 	my $future = Future::IO::Impl::Uring::_Future->new;
 	my $buffer = "\0" x $length;
-	my $id = $ring->read($fh, $buffer, -1, IOSQE_ASYNC, sub($res, $flags) {
+	my $id = $ring->read($fh, $buffer, -1, 0, sub($res, $flags) {
 		if ($res > 0) {
 			$future->done($res == $length ? $buffer : substr($buffer, 0, $res));
 		} elsif ($res == 0) {
@@ -164,7 +150,7 @@ sub sysread($self, $fh, $length) {
 
 sub syswrite($self, $fh, $buffer) {
 	my $future = Future::IO::Impl::Uring::_Future->new;
-	my $id = $ring->write($fh, $buffer, -1, IOSQE_ASYNC, sub($res, $flags) {
+	my $id = $ring->write($fh, $buffer, -1, 0, sub($res, $flags) {
 		if ($res >= 0) {
 			$future->done($res);
 		} else {
@@ -218,7 +204,7 @@ Future::IO::Impl::Uring - A Future::IO implementation for IO::Uring
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 DESCRIPTION
 
@@ -235,6 +221,8 @@ loaded, and it will provide the C<Future::IO> implementation methods:
 
 It requires Linux kernel 6.7 or higher to function.
 
+If L<Future::Uring> has been loaded before this module is, they will share their backend.
+
 =head1 AUTHOR
 
 Leon Timmermans <fawaka@gmail.com>
